-- Preparation:

USE Playground;

-- Demo tables for some of the demos
DROP TABLE IF EXISTS dbo.Fruits;
DROP TABLE IF EXISTS dbo.Colors;

CREATE TABLE dbo.Fruits
    (FruitNum  int         NOT NULL,
     FruitName varchar(35) NOT NULL);
CREATE TABLE dbo.Colors
    (ColorNum  int         NOT NULL,
     ColorName varchar(35) NOT NULL,
     ColorCode binary(3)   NOT NULL);

CREATE CLUSTERED INDEX cix_Fruits_FruitNum
ON dbo.Fruits(FruitNum);
CREATE NONCLUSTERED INDEX cix_Fruits_FruitName
ON dbo.Fruits(FruitName);

CREATE CLUSTERED INDEX cix_Colors_ColorNum
ON dbo.Colors(ColorNum);
CREATE NONCLUSTERED INDEX cix_Colors_ColorName
ON dbo.Colors(ColorName);

INSERT INTO dbo.Fruits (FruitNum, FruitName)
VALUES (1, 'Apple'),
       (2, 'Lime'),
       (2, 'Orange'),
       (3, 'Cherry'),
       (3, 'Melon');

INSERT INTO dbo.Colors (ColorNum, ColorName, ColorCode)
VALUES (0, 'Black',  0x000000),
       (1, 'Red',    0xFF0000),
       (2, 'Orange', 0xFFA500),
       (2, 'Yellow', 0xFFFF00),
       (4, 'Blue',   0x0000FF),
       (5, 'White',  0xFFFFFF);

/*
-- Some of the demos use this demo data instead:
INSERT INTO dbo.Fruits (FruitNum, FruitName)
VALUES (1, 'Apple'),
       (3, 'Cherry'),
       (2, 'Lime'),
       (3, 'Melon'),
       (2, 'Orange'),
       (5, 'Banana');

INSERT INTO dbo.Colors (ColorNum, ColorName, ColorCode)
VALUES (4, 'Blue',   0x0000FF),
       (2, 'Orange', 0xFFA500),
       (1, 'Red',    0xFF0000),
       (2, 'Yellow', 0xFFFF00),
       (5, 'Green',  0x008000),
       (6, 'Cyan',   0x00FFFF),
       (6, 'Teal',   0x008080);
*/


-- Demo 1:

USE Playground;

DROP TABLE IF EXISTS dbo.Small, dbo.Large;

CREATE TABLE dbo.Small
    (KeyCol int NOT NULL PRIMARY KEY);
CREATE TABLE dbo.Large
    (KeyCol int NOT NULL PRIMARY KEY);

INSERT INTO dbo.Small (KeyCol)
SELECT gs.value
FROM   GENERATE_SERIES(1, 3) AS gs;

INSERT INTO dbo.Large (KeyCol)
SELECT gs.value
FROM   GENERATE_SERIES(1, 10) AS gs;

-- Second window:

SELECT     *
FROM       dbo.Small AS s
INNER JOIN dbo.Large AS l
   ON      l.KeyCol = s.KeyCol
OPTION (MERGE JOIN, FORCE ORDER);

SELECT     *
FROM       dbo.Large AS l
INNER JOIN dbo.Small AS s
   ON      l.KeyCol = s.KeyCol
OPTION (MERGE JOIN, FORCE ORDER);

-- Third window:

SELECT          *
FROM            dbo.Small AS s
LEFT OUTER JOIN dbo.Large AS l
  ON            l.KeyCol = s.KeyCol
OPTION (MERGE JOIN, FORCE ORDER);

SELECT          *
FROM            dbo.Large AS l
LEFT OUTER JOIN dbo.Small AS s
  ON            l.KeyCol = s.KeyCol
OPTION (MERGE JOIN, FORCE ORDER);


-- Demo 2:

USE Playground;

SELECT          f.FruitNum,
                f.FruitName,
                c.ColorNum,
                c.ColorName,
                c.ColorCode
FROM            dbo.Fruits AS f
LEFT OUTER JOIN dbo.Colors AS c
  ON            c.ColorNum = f.FruitNum
OPTION (MERGE JOIN, FORCE ORDER);


-- Demo 3:

USE Playground;

-- Table dbo.Small is the same as in first demo
DROP TABLE IF EXISTS dbo.Large2;

CREATE TABLE dbo.Large2
    (ValueCol int NOT NULL,
     INDEX ix_Large2_a (ValueCol));

INSERT INTO dbo.Large2 (ValueCol)
VALUES (1), (1), (2), (2),
       (3), (3), (3), (3),
       (4), (4), (5), (5);

-- Second window

SELECT           s.KeyCol
FROM             dbo.Small  AS s
INNER MERGE JOIN dbo.Large2 AS l
   ON            l.ValueCol = s.KeyCol;

SELECT s.KeyCol
FROM   dbo.Small AS s
WHERE  EXISTS
    (SELECT * FROM dbo.Large2 AS l WHERE l.ValueCol = s.KeyCol)
OPTION (MERGE JOIN, HASH GROUP);


-- Demo 4:

USE AdventureWorks2017;

-- This data is returned in order of ascending SalesOrderID
SELECT   SalesOrderID, CustomerID, DueDate, SubTotal
FROM     Sales.SalesOrderHeader
WHERE    CustomerID = 11200;

-- This data is also returned in order of ascending SalesOrderID
SELECT   SalesOrderID, CustomerID, DueDate, SubTotal
FROM     Sales.SalesOrderHeader
WHERE    CustomerID = 11300;

-- Concatenate these sets, and the result is no longer ordered by SalesOrderID
SELECT   SalesOrderID, CustomerID, DueDate, SubTotal
FROM     Sales.SalesOrderHeader
WHERE    CustomerID = 11200
UNION ALL
SELECT   SalesOrderID, CustomerID, DueDate, SubTotal
FROM     Sales.SalesOrderHeader
WHERE    CustomerID = 11300;


-- Demo 5:

USE Playground;

SELECT KeyCol
FROM   dbo.Small
UNION
SELECT ValueCol
FROM   dbo.Large2;


-- Demo 6:

USE AdventureWorks2017;

SELECT     sod.SalesOrderID,
           sod.OrderQty,
           sod.ProductID,
           soh.CustomerID
FROM       Sales.SalesOrderDetail AS sod
INNER JOIN Sales.SalesOrderHeader AS soh
   ON      soh.SalesOrderID = sod.SalesOrderID;

-- Second window:

SELECT     sod.SalesOrderID,
           sod.OrderQty,
           sod.ProductID,
           soh.CustomerID
FROM       Sales.SalesOrderDetail AS sod
INNER JOIN Sales.SalesOrderHeader AS soh
   ON      soh.CustomerID = sod.SalesOrderID;

SELECT     sod.SalesOrderID,
           sod.OrderQty,
           sod.ProductID,
           soh.CustomerID
FROM       Sales.SalesOrderDetail AS sod
INNER JOIN Sales.SalesOrderHeader AS soh
   ON      soh.CustomerID = sod.SalesOrderID
OPTION (FORCE ORDER);


-- Demo 7:

USE Playground;

SELECT           f.FruitNum,
                 f.FruitName,
                 c.ColorNum,
                 c.ColorName
FROM             dbo.Fruits AS f
INNER MERGE JOIN dbo.Colors AS c
   ON            c.ColorNum = f.FruitNum;


-- Demo 8:

USE AdventureWorks2017;

SELECT * INTO #SalesOrderHeader FROM Sales.SalesOrderHeader;
SELECT * INTO #SalesOrderDetail FROM Sales.SalesOrderDetail;

SELECT     soh.SalesOrderID,
           soh.OrderDate,
           sod.OrderQty,
           sod.ProductID
FROM       #SalesOrderHeader AS soh
INNER JOIN #SalesOrderDetail AS sod
   ON      sod.SalesOrderID = soh.SalesOrderID
OPTION (MERGE JOIN, MAXDOP 1);

SELECT     soh.SalesOrderID,
           soh.OrderDate,
           sod.OrderQty,
           sod.ProductID
FROM       #SalesOrderHeader AS soh
INNER JOIN #SalesOrderDetail AS sod
   ON      sod.SalesOrderID = soh.SalesOrderID
OPTION (MERGE JOIN, MAXDOP 1, FORCE ORDER);


-- Demo 9:

USE AdventureWorks2017;

SELECT          d.Name, e.BirthDate
FROM            HumanResources.Department AS d
FULL OUTER JOIN HumanResources.Employee   AS e
           ON   e.BusinessEntityID         > d.DepartmentID;
