-- Demo 1:

USE AdventureWorks2017;

SELECT soh.SalesOrderID,
       sod.SalesOrderID
FROM   Sales.SalesOrderDetail  AS sod
INNER JOIN
       (SELECT   TOP (19000) *
        FROM     Sales.SalesOrderHeader
        ORDER BY SalesOrderID) AS soh
   ON sod.SalesOrderID = soh.SalesOrderID
OPTION (FORCE ORDER);

SELECT soh.SalesOrderID,
       sod.SalesOrderID
FROM   Sales.SalesOrderDetail  AS sod
LEFT OUTER JOIN
       (SELECT   TOP (19000) *
        FROM     Sales.SalesOrderHeader
        ORDER BY SalesOrderID) AS soh
  ON sod.SalesOrderID = soh.SalesOrderID
OPTION (FORCE ORDER);


-- Demo 2:

USE Playground;

SELECT          c.ColorNum,
                c.ColorName,
                f.FruitNum,
                f.FruitName
FROM            dbo.Colors AS c
FULL OUTER JOIN dbo.Fruits AS f
  ON            c.ColorNum = f.FruitNum
OPTION (HASH JOIN, FORCE ORDER);


-- Demo 3:

USE AdventureWorks2017;

SELECT soh.SalesOrderID,
       soh.SalesPersonID
FROM   Sales.SalesOrderHeader AS soh
WHERE  EXISTS
    (SELECT *
     FROM
            (SELECT   TOP (19000) *
             FROM     Sales.SalesOrderDetail AS sod
             ORDER BY SalesOrderID) AS sod
     WHERE  sod.SalesOrderID = soh.SalesOrderID)
OPTION (HASH JOIN, FORCE ORDER, HASH GROUP);


-- Demo 4:

USE AdventureWorks2017;

SELECT soh.SalesOrderID
FROM   Sales.SalesOrderHeader AS soh
WHERE  soh.SalesPersonID IS NULL
AND    EXISTS
    (SELECT *
     FROM
            (SELECT   TOP (19000) *
             FROM     Sales.SalesOrderDetail AS sod
             ORDER BY SalesOrderID) AS sod
     WHERE  sod.SalesOrderID = soh.SalesOrderID)
OPTION (HASH JOIN, HASH GROUP);


-- Demo 5:

USE AdventureWorks2017;

SELECT soh.ModifiedDate
FROM   Sales.SalesOrderHeader AS soh
UNION
SELECT sod.ModifiedDate
FROM   Sales.SalesOrderDetail AS sod;

-- Second window

SELECT Val
FROM   (VALUES (0),(3),(6),(9),(12)) AS v(Val)
UNION
SELECT Val
FROM   (VALUES (0),(2),(4),(6),(8),(10),(12)) AS v(Val)
OPTION (HASH UNION);


-- Demo 6:

USE AdventureWorks2017;

SELECT sod.SpecialOfferID
FROM   Sales.SalesOrderDetail                          AS sod
INNER JOIN
       (SELECT TOP (50) * FROM Sales.SalesOrderHeader) AS soh
   ON soh.SalesPersonID = sod.SpecialOfferID
OPTION (FORCE ORDER);

SELECT DISTINCT sod.SpecialOfferID
FROM   Sales.SalesOrderDetail AS sod;

SELECT sod.SpecialOfferID
FROM   Sales.SalesOrderDetail AS sod
UNION
SELECT soh.SalesPersonID
FROM   Sales.SalesOrderHeader AS soh
OPTION (HASH UNION);


-- Demo 7:

USE AdventureWorks2017;

SET STATISTICS IO ON;

DECLARE @n int = 50000;

SELECT soh.SalesOrderID,
       soh.OrderDate,
       soh.DueDate,
       sod.OrderQty,
       sod.ProductID,
       sod.UnitPrice
FROM   Sales.SalesOrderHeader  AS soh
INNER JOIN
       (SELECT   TOP (@n) SalesOrderID,
                          OrderQty,
                          ProductID,
                          UnitPrice
        FROM     Sales.SalesOrderDetail
        ORDER BY SalesOrderID) AS sod
   ON sod.SalesOrderID = soh.SalesOrderID
OPTION (HASH JOIN, OPTIMIZE FOR (@n = 200));


-- Demo 8:

USE AdventureWorks2017;

WITH SalesOrders
  AS (SELECT     soh.SalesOrderID,
                 soh.OrderDate,
                 sp.SalesQuota,
                 sp.Bonus
      FROM       Sales.SalesPerson      AS sp
      INNER JOIN Sales.SalesOrderHeader AS soh
         ON      sp.BusinessEntityID = soh.SalesPersonID),
     SalesLines
  AS (SELECT     sod.SalesOrderID,
                 sod.SalesOrderDetailID,
                 p.Name
      FROM       Production.Product     AS p
      INNER JOIN Sales.SalesOrderDetail AS sod
         ON      p.ProductID = sod.ProductID)
SELECT     *
FROM       SalesOrders AS so
INNER JOIN SalesLines  AS sl
   ON      sl.SalesOrderID = so.SalesOrderID
ORDER BY   sl.SalesOrderID,
           sl.SalesOrderDetailID
OPTION (FORCE ORDER, HASH JOIN);
